@@ -41,7 +41,8 @@ TDomainMapping::TDomainMapping(TDomain *dom)
 TDomain::TDomain()
 : classVar((TVariable *)NULL),
   attributes(mlnew TVarList()),
-  variables(mlnew TVarList()), 
+  variables(mlnew TVarList()),
+  classVars(mlnew TVarList()),
   version(++domainVersion),
   lastDomain(knownDomains.end()),
   destroyNotifiers()
@@ -52,6 +53,7 @@ TDomain::TDomain(const TVarList &vl)
 : classVar(vl.size() ? vl.back() : PVariable()),
   attributes(mlnew TVarList(vl)),
   variables(mlnew TVarList(vl)),
+  classVars(mlnew TVarList()),
   version(++domainVersion),
   lastDomain(knownDomains.end()),
   destroyNotifiers()
@@ -64,6 +66,7 @@ TDomain::TDomain(PVariable va, const TVarList &vl)
 : classVar(va),
   attributes(mlnew TVarList(vl)),
   variables(mlnew TVarList(vl)),
+  classVars(mlnew TVarList()),
   version(++domainVersion),
   lastDomain(knownDomains.end()),
   destroyNotifiers()
@@ -78,6 +81,7 @@ TDomain::TDomain(const TDomain &old)
   classVar(old.classVar), 
   attributes(mlnew TVarList(old.attributes.getReference())),
   variables(mlnew TVarList(old.variables.getReference())), 
+  classVars(mlnew TVarList(old.classVars.getReference())),
   metas(old.metas),
   version(++domainVersion),
   knownDomains(),  // don't copy, unless you want to mess with the notifiers..
@@ -124,7 +128,8 @@ void TDomain::domainHasChanged()
 
 void TDomain::afterSet(const char *name)
 { 
-  if (!strcmp(name, "classVar")) {
+  if (!strcmp(name, "class_var") 
+      || !strcmp(name, "classVar")) {
     if (attributes->size()==variables->size())
       variables->push_back(classVar);
     else
@@ -202,13 +207,17 @@ bool TDomain::delVariable(PVariable var)
 
 int TDomain::getVarNum(PVariable var, bool throwExc) const
 { int pos = 0;
-  for(TVarList::const_iterator vi(variables->begin()), ve(variables->end()); vi!=ve; vi++, pos++)
+  TVarList::const_iterator vi, ve;
+  for(vi = variables->begin(), ve = variables->end(); vi!=ve; vi++, pos++)
     if (*vi == var)
       return pos;
+  for(vi = classVars->begin(), ve = classVars->end(); vi != ve; vi++, pos++)
+      if (*vi == var)
+            return pos;
 
   pos = getMetaNum(var, false);
   if ((pos == ILLEGAL_INT) && throwExc)
-    raiseError("attribute '%s' not found", var->name.c_str());
+    raiseError("attribute '%s' not found", var->get_name().c_str());
 
   return pos;
 }
@@ -216,9 +225,13 @@ int TDomain::getVarNum(PVariable var, bool throwExc) const
 
 int TDomain::getVarNum(const string &name, bool throwExc) const
 { int pos = 0;
-  for(TVarList::const_iterator vi(variables->begin()), ve(variables->end()); vi!=ve; vi++, pos++)
-    if ((*vi)->name == name)
+  TVarList::const_iterator vi, ve;
+  for(vi = variables->begin(), ve = variables->end(); vi!=ve; vi++, pos++)
+    if ((*vi)->get_name()== name)
       return pos;
+  for(vi = classVars->begin(), ve = classVars->end(); vi != ve; vi++, pos++)
+        if ((*vi)->get_name() == name)
+            return pos;
 
   pos = getMetaNum(name, false);
   if ((pos == ILLEGAL_INT) && throwExc)
@@ -232,16 +245,17 @@ PVariable TDomain::getVar(int num, bool throwExc) const
 { checkProperty(variables);
   
   if (num>=0) {
-    if (num>=int(variables->size()))
+      if (num < variables->size())
+          return variables->at(num);
+      if (num - variables->size() < classVars->size())
+          return classVars->at(num - variables->size());
       if (throwExc)
         if (!variables->size())
           raiseError("no attributes in domain");
         else
-          raiseError("index %i out of range 0-%i", num, variables->size()-1);
+          raiseError("index %i out of range", num);
       else
         return PVariable();
-
-    return variables->at(num);
   }
   else {
     const_ITERATE(TMetaVector, mi, metas)
@@ -262,16 +276,19 @@ PVariable TDomain::getVar(int num, bool throwExc)
 { checkProperty(variables);
   
   if (num>=0) {
-    if (num>=int(variables->size()))
+      if (num < variables->size())
+          return variables->at(num);
+
+      if (num - variables->size() < classVars->size())
+          return classVars->at(num - variables->size());
+
       if (throwExc)
         if (!variables->size())
           raiseError("no attributes in domain");
         else
-          raiseError("index %i out of range 0-%i", num, variables->size()-1);
+          raiseError("index %i out of range", num);
       else
         return PVariable();
-
-    return variables->at(num);
   }
   else {
     ITERATE(TMetaVector, mi, metas)
@@ -289,13 +306,18 @@ PVariable TDomain::getVar(int num, bool throwExc)
 
 
 PVariable TDomain::getVar(const string &name, bool takeMetas, bool throwExc)
-{ PITERATE(TVarList, vi, variables)
-    if ((*vi)->name==name)
+{ 
+  PITERATE(TVarList, vi, variables) {
+    if ((*vi)->get_name()==name)
       return *vi;
-
+  }
+  PITERATE(TVarList, vi2, classVars) {
+    if ((*vi2)->get_name()==name)
+      return *vi2;
+  }
   if (takeMetas)
     ITERATE(TMetaVector, mi, metas)
-      if ((*mi).variable->name==name)
+      if ((*mi).variable->get_name()==name)
         return (*mi).variable;
 
   if (throwExc)
@@ -306,13 +328,18 @@ PVariable TDomain::getVar(const string &name, bool takeMetas, bool throwExc)
 
 
 PVariable TDomain::getVar(const string &name, bool takeMetas, bool throwExc) const
-{ const_PITERATE(TVarList, vi, variables)
-    if ((*vi)->name==name)
+{ const_PITERATE(TVarList, vi, variables) {
+    if ((*vi)->get_name()==name)
       return *vi;
+  }
+  const_PITERATE(TVarList, vi2, classVars) {
+    if ((*vi2)->get_name()==name)
+      return *vi2;
+  }
 
   if (takeMetas)
     const_ITERATE(TMetaVector, mi, metas)
-      if ((*mi).variable->name==name)
+      if ((*mi).variable->get_name()==name)
         return (*mi).variable;
 
   if (throwExc)
@@ -324,7 +351,7 @@ PVariable TDomain::getVar(const string &name, bool takeMetas, bool throwExc) con
 
 const TMetaDescriptor *TDomain::getMetaDescriptor(const string &wname, bool throwExc) const
 { const_ITERATE(TMetaVector, mi, metas)
-    if ((*mi).variable->name==wname)
+    if ((*mi).variable->get_name()==wname)
       return &*mi;
 
   if (throwExc)
@@ -339,7 +366,7 @@ const TMetaDescriptor *TDomain::getMetaDescriptor(PVariable var, bool throwExc)
       return &*mi;
 
   if (throwExc)
-    raiseError("meta attribute '%s' not found", var->name.c_str());
+    raiseError("meta attribute '%s' not found", var->get_name().c_str());
   
   return NULL;
 }
@@ -370,7 +397,7 @@ long TDomain::getMetaNum(PVariable var, bool throwExc) const
       return (*mi).id;
 
   if (throwExc)
-    raiseError("meta attribute '%s' not found", var->name.c_str());
+    raiseError("meta attribute '%s' not found", var->get_name().c_str());
 
   return ILLEGAL_INT;
 }
@@ -402,7 +429,7 @@ PVariable TDomain::getMetaVar(const int &idx, bool throwExc) const
 
 PVariable TDomain::getMetaVar(const string &wname, bool throwExc) const
 { const_ITERATE(TMetaVector, mi, metas)
-    if ((*mi).variable->name==wname)
+    if ((*mi).variable->get_name()==wname)
       return (*mi).variable;
 
   if (throwExc)
@@ -414,7 +441,7 @@ PVariable TDomain::getMetaVar(const string &wname, bool throwExc) const
 
 PVariable TDomain::getMetaVar(const string &wname, bool throwExc)
 { ITERATE(TMetaVector, mi, metas)
-    if ((*mi).variable->name==wname)
+    if ((*mi).variable->get_name()==wname)
       return (*mi).variable;
 
   if (throwExc)
@@ -440,7 +467,7 @@ void TDomain::convert(TExample &dest, const TExample &src, bool filterMetas)
 {
   dest.id = src.id; 
   if (src.domain==this) {
-    int Nv = variables->size();
+    int Nv = variables->size() + classVars->size();
     TExample::iterator de = dest.begin();
     TExample::const_iterator sr = src.begin();
     while(Nv--)
@@ -468,6 +495,13 @@ void TDomain::convert(TExample &dest, const TExample &src, bool filterMetas)
           (*lastDomain).metasNotToCopy.insert(cvi);
       }
 
+      const_PITERATE(TVarList, mvi, classVars) {
+        const int cvi = src.domain->getVarNum(*mvi, false);
+        (*lastDomain).positions.push_back(cvi);
+        if (cvi<0)
+          (*lastDomain).metasNotToCopy.insert(cvi);
+      }
+
       ITERATE(TMetaVector, mvi, metas) {
         const int cvi = src.domain->getVarNum((*mvi).variable, false);
         (*lastDomain).metaPositions.push_back(make_pair(int((*mvi).id), cvi));
@@ -480,11 +514,16 @@ void TDomain::convert(TExample &dest, const TExample &src, bool filterMetas)
     }
 
     // Now, lastDomain points to an appropriate mapping
-    vector<int>::iterator pi((*lastDomain).positions.begin());
-    TVarList::iterator vi(variables->begin());
-
     TExample::iterator deval(dest.begin());
-    for(int Nv = dest.domain->variables->size(); Nv--; pi++, vi++)
+    vector<int>::iterator pi((*lastDomain).positions.begin());
+
+    TVarList::iterator vi;
+    int Nv;
+    
+    for(vi = variables->begin(), Nv = dest.domain->variables->size(); Nv--; pi++, vi++)
+      *(deval++) = (*pi == ILLEGAL_INT) ? (*vi)->computeValue(src) : src[*pi];
+
+    for(vi = classVars->begin(), Nv = dest.domain->classVars->size(); Nv--; pi++, vi++)
       *(deval++) = (*pi == ILLEGAL_INT) ? (*vi)->computeValue(src) : src[*pi];
 
     TMetaVector::iterator mvi(metas.begin());
@@ -578,10 +617,10 @@ PVariable TDomain::hasContinuousAttributes(bool checkClass) const
 
 #include "crc.h"
 
-void TDomain::addToCRC(unsigned long &crc) const
+void TDomain::addToCRC(unsigned int &crc) const
 {
   const_PITERATE(TVarList, vi, variables) {
-    add_CRC((*vi)->name.c_str(), crc);
+    add_CRC((*vi)->get_name().c_str(), crc);
     add_CRC((const unsigned char)(*vi)->varType, crc);
     if ((*vi)->varType == TValue::INTVAR)
       PITERATE(TStringList, vli, dynamic_cast<TEnumVariable &>(vi->getReference()).values)
@@ -591,7 +630,7 @@ void TDomain::addToCRC(unsigned long &crc) const
 
 
 int TDomain::sumValues() const
-{ unsigned long crc;
+{ unsigned int crc;
   INIT_CRC(crc);
   addToCRC(crc);
   FINISH_CRC(crc);
