@@ -131,7 +131,7 @@ PDomain combineDomains(PDomainList sources, TDomainMultiMapping &mapping)
         newDomain->metas.push_back(*mi);
       else
         if (metavar != (*mi).variable)
-          raiseError("Id %i represents two different attributes ('%s' and '%s')", (*mi).id, metavar->name.c_str(), (*mi).variable->name.c_str());
+          raiseError("Id %i represents two different attributes ('%s' and '%s')", (*mi).id, metavar->get_name().c_str(), (*mi).variable->get_name().c_str());
     }
 
   return wdomain;
@@ -241,7 +241,7 @@ bool TDomainDepot::checkDomain(const TDomain *domain,
   TVarList::const_iterator vi(domain->variables->begin());
   TPAttributeDescriptions::const_iterator ai(attributes->begin()), ae(attributes->end());
   for(; ai != ae; ai++, vi++)
-    if (    ((**ai).name != (*vi)->name)
+    if (    ((**ai).name != (*vi)->get_name())
          || ((**ai).varType>0) && ((**ai).varType != (*vi)->varType)
          || (((**ai).varType==PYTHONVAR) && !pythonDeclarationMatches((**ai).typeDeclaration, *vi))
          || !checkValueOrder(*vi, **ai)
@@ -275,6 +275,7 @@ bool TDomainDepot::checkDomain(const TDomain *domain,
 
 
 PDomain TDomainDepot::prepareDomain(TPAttributeDescriptions *attributes, bool hasClass,
+                                    TPAttributeDescriptions *classDescriptions,
                                     TPAttributeDescriptions *metas, const int createNewOn,
                                     vector<int> &status, vector<pair<int, int> > &metaStatus)
 { 
@@ -286,7 +287,7 @@ PDomain TDomainDepot::prepareDomain(TPAttributeDescriptions *attributes, bool ha
     attrList.push_back(makeVariable(**ai, tStatus, createNewOn));
     status.push_back(tStatus);
   }
-
+  
   PDomain newDomain;
   PVariable classVar;
   if (hasClass) {
@@ -296,6 +297,12 @@ PDomain TDomainDepot::prepareDomain(TPAttributeDescriptions *attributes, bool ha
   
   newDomain = mlnew TDomain(classVar, attrList);
 
+  if (classDescriptions) {
+      PITERATE(TPAttributeDescriptions, ai, classDescriptions) {
+          newDomain->classVars->push_back(makeVariable(**ai, tStatus, createNewOn));
+      }
+  }
+
   metaStatus.clear();
   if (metas)
     PITERATE(TPAttributeDescriptions, mi, metas) {
@@ -314,7 +321,7 @@ PVariable TDomainDepot::createVariable_Python(const string &typeDeclaration, con
 {
   if (typeDeclaration.size() == 6) {
     TPythonVariable *var = mlnew TPythonVariable();
-    var->name = name;
+    var->set_name(name);
     return var;
   }
 
@@ -354,13 +361,13 @@ PVariable TDomainDepot::createVariable_Python(const string &typeDeclaration, con
 
   if (!PyObject_TypeCheck(var, (PyTypeObject *)&PyOrVariable_Type)) {
     Py_DECREF(var);
-    ::raiseErrorWho("makeVariable", "PythonVariable's constructor is expected to return a 'PythonVariable', not '%s'", var->ob_type->tp_name);
+    ::raiseErrorWho("make_variable", "PythonVariable's constructor is expected to return a 'PythonVariable', not '%s'", var->ob_type->tp_name);
   }
 
   PVariable pvar = GCPtr<TOrange>((TPyOrange *)var, true);
   Py_DECREF(var);
 
-  pvar->name = name;
+  pvar->set_name(name);
   return pvar;
 }
 
@@ -382,7 +389,7 @@ PVariable TDomainDepot::makeVariable(TAttributeDescription &desc, int &status, c
     }
 
     if (!var)
-      ::raiseErrorWho("makeVariable", "unknown type for attribute '%s'", desc.name.c_str());
+      ::raiseErrorWho("make_variable", "unknown type for attribute '%s'", desc.name.c_str());
   }
   
   if (var && desc.ordered)
