@@ -27,10 +27,10 @@
 using namespace std;
 
 #include "getarg.hpp"
+#include "orvector.hpp"
+#include "vars.hpp"
+#include "domain.hpp"
 
-VWRAPPER(StringList)
-VWRAPPER(VarList)
-WRAPPER(Domain)
 class TMetaVector;
 
 #define TDomainList TOrangeVector<PDomain> 
@@ -76,6 +76,7 @@ public:
                           const TPAttributeDescriptions *metas, int *metaIDs = NULL);
 
   PDomain prepareDomain(TPAttributeDescriptions *attributes, bool hasClass,
+                        TPAttributeDescriptions *classDescriptions,
                         TPAttributeDescriptions *metas, const int createNewOn,
                         vector<int> &status, vector<pair<int, int> > &metaStatus);
 
