@@ -32,12 +32,14 @@
 #include "distvars.hpp"
 #include "examplegen.hpp"
 
-#include "filter.ppp"
-
+#include "filter.hpp"
 
 DEFINE_TOrangeVector_classDescription(PValueFilter, "TValueFilterList", true, ORANGE_API)
 DEFINE_TOrangeVector_classDescription(PFilter, "TFilterList", true, ORANGE_API)
 
+#include "filter.ppp"
+
+
 // Sets the negate field (default is false)
 TFilter::TFilter(bool anegate, PDomain dom) 
 : negate(anegate),
@@ -515,7 +517,7 @@ void TFilter_values::removeCondition(PVariable var)
   TValueFilterList::iterator condi = findCondition(var, 0, position);
 
   if (condi==conditions->end())
-    raiseError("there is no condition on value of '%s' in the filter", var->name.c_str());
+    raiseError("there is no condition on value of '%s' in the filter", var->get_name().c_str());
 
   conditions->erase(condi);
 }
