@@ -40,11 +40,13 @@
 #include "learn.hpp"
 
 #include <string>
-#include "preprocessors.ppp"
+#include "preprocessors.hpp"
 
 DEFINE_TOrangeMap_classDescription(TOrangeMap_KV, PVariable, PValueFilter, "VariableFilterMap")
 DEFINE_TOrangeMap_classDescription(TOrangeMap_K, PVariable, float, "VariableFloatMap")
 
+#include "preprocessors.ppp"
+
 #ifdef _MSC_VER
   #pragma warning (disable : 4100) // unreferenced local parameter (macros name all arguments)
 #endif
@@ -98,7 +100,7 @@ PExampleGenerator TPreprocessor_ignore::operator()(PExampleGenerator gen, const
       if (*vi == outDomain->classVar)
         outDomain->removeClass();
       else
-        raiseError("attribute '%s' not found", (*vi)->name.c_str());
+        raiseError("attribute '%s' not found", (*vi)->get_name().c_str());
 
   newWeight = weightID;
   return PExampleGenerator(mlnew TExampleTable(outDomain, gen));
@@ -284,7 +286,7 @@ PExampleGenerator TPreprocessor_shuffle::operator()(PExampleGenerator gen, const
   PITERATE(TVarList, vi, attributes) {
     const int idx = gen->domain->getVarNum(*vi, false);
     if (idx == ILLEGAL_INT)
-      raiseError("attribute '%s' not found", (*vi)->name.c_str());
+      raiseError("attribute '%s' not found", (*vi)->get_name().c_str());
     indices.push_back(idx);
   }
     
@@ -413,7 +415,7 @@ PExampleGenerator TPreprocessor_addNoise::operator()(PExampleGenerator gen, cons
     if (*pi > 0.0) {
       const PVariable &var = domain.variables->at(idx);
       if (var->varType != TValue::INTVAR)
-        raiseError("Cannot add noise to non-discrete attribute '%s'", var->name.c_str());
+        raiseError("Cannot add noise to non-discrete attribute '%s'", var->get_name().c_str());
       addNoise(idx, *pi, makerind, table);
     }
   }
@@ -460,7 +462,7 @@ PExampleGenerator TPreprocessor_addGaussianNoise::operator()(PExampleGenerator g
     PITERATE(TVariableFloatMap, vi, deviations) {
       PVariable var = (*vi).first;
       if (var->varType != TValue::FLOATVAR)
-        raiseError("attribute '%s' is not continuous", var->name.c_str());
+        raiseError("attribute '%s' is not continuous", var->get_name().c_str());
 
       const int pos = domain.getVarNum(var);
       ps.push_back(pair<int, float>(pos, (*vi).second));
@@ -552,7 +554,7 @@ PExampleGenerator TPreprocessor_addGaussianClassNoise::operator()(PExampleGenera
   if (!classVar)
     raiseError("Class-less domain");
   if (classVar->varType != TValue::FLOATVAR)
-    raiseError("Class '%s' is not continuous", gen->domain->classVar->name.c_str());
+    raiseError("Class '%s' is not continuous", gen->domain->classVar->get_name().c_str());
 
   newWeight = weightID;
 
@@ -864,7 +866,7 @@ PExampleGenerator TPreprocessor_discretize::operator()(PExampleGenerator gen, co
   
   const TDomain &domain = gen->domain.getReference();
   
-  const_PITERATE(TVarList, vi, domain.variables)
+  const_PITERATE(TVarList, vi, domain.attributes)
     if (   ((*vi)->varType == TValue::FLOATVAR)
         && (   !attributes || !attributes->size() 
             || exists(attributes->begin(), attributes->end(), *vi))) {
@@ -878,9 +880,21 @@ PExampleGenerator TPreprocessor_discretize::operator()(PExampleGenerator gen, co
       newDomain->attributes->push_back(*vi);
     }
 
-  if (gen->domain->classVar) {
-    newDomain->classVar = newDomain->variables->back();
-    newDomain->attributes->erase(newDomain->attributes->end()-1);
+  // classVar discretization
+  if (domain.classVar){
+	  if (domain.classVar->varType == TValue::FLOATVAR
+		  && (   !attributes || !attributes->size()
+		             || exists(attributes->begin(), attributes->end(), domain.classVar))
+		  && discretizeClass) {
+		       PVariable evar = method->operator()(gen, domain.classVar);
+		       newDomain->variables->push_back(evar);
+		       newDomain->classVar = evar;
+		       discretized.push_back(evar);
+		     }
+		     else {
+		       newDomain->variables->push_back(domain.classVar);
+		       newDomain->classVar = domain.classVar;
+		     }
   }
   
   if (attributes)
@@ -888,9 +902,9 @@ PExampleGenerator TPreprocessor_discretize::operator()(PExampleGenerator gen, co
       if (!exists(discretized.begin(), discretized.end(), *ai)) {
         long varNum = domain.getVarNum(*ai);
         if (varNum == ILLEGAL_INT)
-          raiseError("Attribute '%s' is not found", (*ai)->name.c_str());
+          raiseError("Attribute '%s' is not found", (*ai)->get_name().c_str());
         else if ((varNum >= 0) || ((*ai)->varType != TValue::FLOATVAR))
-          raiseError("Attribute '%s' is not continuous", (*ai)->name.c_str());
+          raiseError("Attribute '%s' is not continuous", (*ai)->get_name().c_str());
         else {
           PVariable evar = method->operator()(gen, *ai);
           TMetaDescriptor ndsc(varNum, evar);
@@ -1102,4 +1116,4 @@ PExampleTable TTableAverager::operator()(PExampleGeneratorList tables) const
         }
         
     return wtable;
-}
\ No newline at end of file
+}
