@@ -120,6 +120,11 @@ bool TTabDelimExampleGenerator::readExample(TFileExampleIteratorData &fei, TExam
   TVarList::iterator vi(domain->attributes->begin());
   vector<string>::iterator ai(atoms.begin());
   TIntList::iterator si(attributeTypes->begin()), se(attributeTypes->end());
+  TIntList::iterator cb, cp, ce;
+  if (classPoses) {
+      cb = cp = classPoses->begin();
+      ce = classPoses->end();
+  }
   int pos=0;
   for (; (si!=se); pos++, si++, ai++) {
     if (*si) { // if attribute is not to be skipped and is not a basket
@@ -139,6 +144,11 @@ bool TTabDelimExampleGenerator::readExample(TFileExampleIteratorData &fei, TExam
             domain->classVar->filestr2val(valstr, cval, exam);
             exam.setClass(cval);
           }
+          else if (classPoses && (cp != ce) && (pos == *cp)) {
+              const int ind = cp - cb;
+              domain->classVars->at(ind)->filestr2val(valstr, exam.values_end[ind], exam);
+              cp++;
+          }
           else { // if this is a normal value
             (*vi++)->filestr2val(valstr, *ei++, exam);
           }
@@ -169,6 +179,12 @@ bool TTabDelimExampleGenerator::readExample(TFileExampleIteratorData &fei, TExam
 
   if (pos==classPos) // if class is the last value in the line, it is set here
     domain->classVar->filestr2val(ai==atoms.end() ? "?" : *(ai++), exam[domain->variables->size()-1], exam);
+  /* I'm not sure that this is needed; this code is a mess but I don't wish to
+     waste time studying it since we are moving to 3.0 */
+  else if (classPoses && (cp != ce) && (pos == *cp)) {
+    const int ind = cp - cb;
+    domain->classVars->at(ind)->filestr2val(ai==atoms.end() ? "?" : *(ai++), exam.values_end[ind], exam);
+  }
 
   while ((ai!=atoms.end()) && !(*ai).length()) ai++; // line must be empty from now on
 
@@ -277,7 +293,7 @@ char *TTabDelimExampleGenerator::mayBeTabFile(const string &stem)
         return res;
       }
 
-      static const char *legalDirects[] = {"s", "skip","i", "ignore", "c", "class", "m", "meta", NULL};
+      static const char *legalDirects[] = {"s", "skip","i", "ignore", "c", "class", "multiclass", "m", "meta", NULL};
       string &direct = args.direct.front();
       const char **lc = legalDirects;
       while(*lc && strcmp(*lc, direct.c_str()))
@@ -318,8 +334,13 @@ PDomain TTabDelimExampleGenerator::readDomain(const string &stem, const bool aut
   scanAttributeValues(stem, descriptions);
   
   TIntList::iterator ati(attributeTypes->begin());
-  TDomainDepot::TPAttributeDescriptions attributeDescriptions, metaDescriptions;
+  TDomainDepot::TPAttributeDescriptions attributeDescriptions, metaDescriptions, classDescriptions;
   int ind = 0, lastRegular = -1;
+  TIntList::const_iterator cp, ce;
+  if (classPoses) {
+      cp = classPoses->begin();
+      ce = classPoses->end();
+  }
 
   for(TDomainDepot::TAttributeDescriptions::iterator adi(descriptions.begin()), ade(descriptions.end()); adi != ade; adi++, ati++, ind++) {
     if (!*ati)
@@ -350,7 +371,10 @@ PDomain TTabDelimExampleGenerator::readDomain(const string &stem, const bool aut
     
     if (*ati == 1)
       metaDescriptions.push_back(&*adi);
-      
+    else if (classPoses && (cp != ce) && (*cp == ind)) {
+        classDescriptions.push_back(&*adi);
+        cp++;
+    }
     else if ((classPos != ind) && (basketPos != ind)) {
       attributeDescriptions.push_back(&*adi);
       lastRegular = ind;
@@ -375,7 +399,7 @@ PDomain TTabDelimExampleGenerator::readDomain(const string &stem, const bool aut
     return sourceDomain;
   }
 */
-  PDomain newDomain = domainDepot.prepareDomain(&attributeDescriptions, classPos>-1, &metaDescriptions, createNewOn, status, metaStatus);
+  PDomain newDomain = domainDepot.prepareDomain(&attributeDescriptions, classPos>-1, &classDescriptions, &metaDescriptions, createNewOn, status, metaStatus);
 
   vector<pair<int, int> >::const_iterator mid(metaStatus.begin());
   PITERATE(TIntList, ii, attributeTypes)
@@ -434,7 +458,7 @@ int TTabDelimExampleGenerator::detectAttributeType(TDomainDepot::TAttributeDescr
   }
   
   /* Check whether this is a string attribute:
-     - has more at least 20 values
+     - has more than 20 values
      - less than half of the values appear more than once */
   if ((status==0) && (desc.values.size() > 20)) {
       int more2 = 0;
@@ -502,19 +526,23 @@ void TTabDelimExampleGenerator::scanAttributeValues(const string &stem, TDomainD
     for(di = db, ati = atb, ai = atoms.begin(), ae = atoms.end(); (di != de) && (ai != ae); di++, ai++, ati++) {
       if (!*atb)
         continue;
-        
-      const char *ceni = ai->c_str();
-      if (   !*ceni
-          || !ceni[1] && ((*ceni=='?') || (*ceni=='.') || (*ceni=='~') || (*ceni=='*'))
-          || (*ai == "NA") || (DC && (*ai == DC)) || (DK && (*ai == DK)))
-         continue;
+   
+      //skip the attribute if it is a FLOATVAR or STRINGVAR
+      if ((di->varType != TValue::FLOATVAR) && (di->varType != STRINGVAR)) {
 
-      map<string, int>::iterator vf = di->values.lower_bound(*ai);
-      if ((vf != di->values.end()) && (vf->first == *ai)) {
-        vf->second++;
-      }
-      else {
-        di->values.insert(vf, make_pair(*ai, 1));
+        const char *ceni = ai->c_str();
+        if (   !*ceni
+            || !ceni[1] && ((*ceni=='?') || (*ceni=='.') || (*ceni=='~') || (*ceni=='*'))
+            || (*ai == "NA") || (DC && (*ai == DC)) || (DK && (*ai == DK)))
+           continue;
+
+        map<string, int>::iterator vf = di->values.lower_bound(*ai);
+        if ((vf != di->values.end()) && (vf->first == *ai)) {
+          vf->second++;
+        }
+        else {
+          di->values.insert(vf, make_pair(*ai, 1));
+        }
       }
     }
   }
@@ -605,6 +633,7 @@ void TTabDelimExampleGenerator::readTxtHeader(const string &stem, TDomainDepot::
 void TTabDelimExampleGenerator::readTabHeader(const string &stem, TDomainDepot::TAttributeDescriptions &descs)
 {
   classPos = -1;
+  classPoses = mlnew TIntList;
   basketPos = -1;
   headerLines = 3;
 
@@ -662,6 +691,10 @@ void TTabDelimExampleGenerator::readTabHeader(const string &stem, TDomainDepot::
           classPos = ind;
         }
         
+        else if (direct=="multiclass") {
+          classPoses->push_back(ind);
+        }
+        
         else if ((direct=="m") || (direct=="meta"))
           *attributeType = 1;
       }
@@ -674,7 +707,7 @@ void TTabDelimExampleGenerator::readTabHeader(const string &stem, TDomainDepot::
 
       desc.userFlags = args.unrecognized;
     }
-
+    
     if (!strcmp((*ti).c_str(), "basket")) {
       if (basketPos > -1)
         ::raiseError("multiple basket attributes are defined");
@@ -723,6 +756,9 @@ void TTabDelimExampleGenerator::readTabHeader(const string &stem, TDomainDepot::
         desc.addValue(vals);
     }
   }
+
+  if (!classPoses->size())
+      classPoses = PIntList();
 }
 
 
@@ -795,13 +831,13 @@ int readTabAtom(TFileExampleIteratorData &fei, vector<string> &atoms, bool escap
 
       case '\t':
         atoms.push_back(trim(atom));
-        atom = string();
+        atom.clear();
         break;
 
       case ',':
         if (csv) {
           atoms.push_back(trim(atom));
-          atom = string();
+          atom.clear();
           break;
         }
         // else fallthrough
@@ -857,14 +893,18 @@ void tabDelim_writeExamples(FILE *file, PExampleGenerator rg, char delim, const
 { 
   const TDomain domain = rg->domain.getReference();
   TVarList::const_iterator vb(domain.variables->begin()), vi, ve(domain.variables->end());
+  TVarList::const_iterator cvb(domain.classVars->begin()), cve(domain.classVars->end());
 
   PEITERATE(ex, rg) {
-    vi = vb;
     TExample::const_iterator ri((*ex).begin());
     string st;
     bool ho = false;
 
-    for(; vi!=ve; vi++, ri++) {
+    for(vi=vb; vi!=cve; vi++, ri++) {
+      if (vi==ve) {
+        vi = cvb;
+        if (vi==cve) break;
+      }
       PUTDELIM;
       if (DK && ((*ri).valueType == valueDK))
         fprintf(file, DK);
@@ -881,12 +921,13 @@ void tabDelim_writeExamples(FILE *file, PExampleGenerator rg, char delim, const
     for(mi = mb; mi != me; mi++) {
       if (!(*mi).optional) {
         PUTDELIM;
-        if (DK && ((*ri).valueType == valueDK))
+        const TValue &mval = (*ex)[(*mi).id];
+        if (DK && (mval.valueType == valueDK))
           fprintf(file, DK);
-        else if (DC && ((*ri).valueType == valueDC))
+        else if (DC && (mval.valueType == valueDC))
           fprintf(file, DC);
         else {
-          (*mi).variable->val2filestr((*ex)[(*mi).id], st, *ex);
+          (*mi).variable->val2filestr(mval, st, *ex);
           fprintf(file, "%s", checkCtrl(st.c_str()));
         }
       }
@@ -907,10 +948,10 @@ void tabDelim_writeExamples(FILE *file, PExampleGenerator rg, char delim, const
               fprintf(file, " ");
 
             if (mval.floatV == 1.0)
-              fprintf(file, checkCtrl(var.name.c_str()));
+              fprintf(file, checkCtrl(var.get_name().c_str()));
             else {
               var.val2filestr(mval, st, *ex);
-              fprintf(file, "%s=%s", checkCtrl(var.name.c_str()), checkCtrl(st.c_str()));
+              fprintf(file, "%s=%s", checkCtrl(var.get_name().c_str()), checkCtrl(st.c_str()));
             }
           }
         }
@@ -992,6 +1033,7 @@ void tabDelim_printAttributes(FILE *file, PVariable var, bool needsSpace) {
 void tabDelim_writeDomainWithoutDetection(FILE *file, PDomain dom, char delim, bool listDiscreteValues)
 { 
   TVarList::const_iterator vi, vb(dom->variables->begin()), ve(dom->variables->end());
+  TVarList::const_iterator cvi, cvb(dom->classVars->begin()), cve(dom->classVars->end());
   TMetaVector::const_iterator mi, mb(dom->metas.begin()), me(dom->metas.end());
 
   bool ho = false;
@@ -1000,7 +1042,11 @@ void tabDelim_writeDomainWithoutDetection(FILE *file, PDomain dom, char delim, b
   // First line: attribute names
   for(vi = vb; vi!=ve; vi++) {
     PUTDELIM;
-    fprintf(file, "%s", checkCtrl((*vi)->name.c_str()));
+    fprintf(file, "%s", checkCtrl((*vi)->get_name().c_str()));
+  }
+  for(cvi = cvb; cvi!=cve; cvi++) {
+    PUTDELIM;
+    fprintf(file, "%s", checkCtrl((*cvi)->get_name().c_str()));
   }
   for(mi = mb; mi!=me; mi++) {
     if (mi->optional) {
@@ -1009,7 +1055,7 @@ void tabDelim_writeDomainWithoutDetection(FILE *file, PDomain dom, char delim, b
     }
     else {
       PUTDELIM;
-      fprintf(file, "%s", checkCtrl((*mi).variable->name.c_str()));
+      fprintf(file, "%s", checkCtrl((*mi).variable->get_name().c_str()));
     }
   }
 
@@ -1027,6 +1073,10 @@ void tabDelim_writeDomainWithoutDetection(FILE *file, PDomain dom, char delim, b
     PUTDELIM;
     printVarType(file, *vi, listDiscreteValues);
   }
+  for(cvi = cvb; cvi!=cve; cvi++) {
+    PUTDELIM;
+    printVarType(file, *cvi, listDiscreteValues);
+  }
   for(mi = mb; mi!=me; mi++) {
     if (mi->optional)
       continue;
@@ -1056,6 +1106,11 @@ void tabDelim_writeDomainWithoutDetection(FILE *file, PDomain dom, char delim, b
     fprintf(file, "class");
     tabDelim_printAttributes(file, dom->classVar, true);
   }
+  for(cvi = cvb; cvi!=cve; cvi++) {
+    PUTDELIM;
+    fprintf(file, "multiclass");
+    tabDelim_printAttributes(file, *cvi, true);
+  }
   for(mi = mb; mi!=me; mi++) {
     if (mi->optional)
       continue;
@@ -1122,12 +1177,12 @@ void tabDelim_writeDomainWithDetection(FILE *file, PDomain dom, char delim)
   bool ho = false;
   const_PITERATE(TVarList, vi, dom->attributes) {
     PUTDELIM;
-    fprintf(file, "%s%s", (tabDelim_checkNeedsD(*vi) ? "D#" : ""), checkCtrl((*vi)->name.c_str()));
+    fprintf(file, "%s%s", (tabDelim_checkNeedsD(*vi) ? "D#" : ""), checkCtrl((*vi)->get_name().c_str()));
   }
   
   if (dom->classVar) {
     PUTDELIM;
-    fprintf(file, "%s%s", (tabDelim_checkNeedsD(dom->classVar) ? "cD#" : "c#"), checkCtrl(dom->classVar->name.c_str()));
+    fprintf(file, "%s%s", (tabDelim_checkNeedsD(dom->classVar) ? "cD#" : "c#"), checkCtrl(dom->classVar->get_name().c_str()));
   }
 
 
@@ -1140,7 +1195,7 @@ void tabDelim_writeDomainWithDetection(FILE *file, PDomain dom, char delim)
     }
     else {
       PUTDELIM;
-      fprintf(file, "%s%s", (tabDelim_checkNeedsD((*mi).variable) ? "mD#" : "m#"), checkCtrl((*mi).variable->name.c_str()));
+      fprintf(file, "%s%s", (tabDelim_checkNeedsD((*mi).variable) ? "mD#" : "m#"), checkCtrl((*mi).variable->get_name().c_str()));
     }
   }
 
