@@ -276,7 +276,9 @@ PClassifier TTreeSplitConstructor_Attribute::operator()(
 
   spentAttribute = bestAttr;
 
-  return mlnew TClassifierFromVarFD(attribute, gen->domain, bestAttr, subsetSizes);
+  TClassifierFromVarFD *cfv = mlnew TClassifierFromVarFD(attribute, gen->domain, bestAttr, subsetSizes);
+  cfv->transformUnknowns = false;
+  return cfv;
 }
 
 
@@ -400,7 +402,7 @@ PClassifier TTreeSplitConstructor_ExhaustiveBinary::operator()(
         const TDistributionVector &distr = *(*dci)->discrete;
 
         if (distr.size()>16)
-          raiseError("'%s' has more than 16 values, cannot exhaustively binarize", gen->domain->attributes->at(thisAttr)->name.c_str());
+          raiseError("'%s' has more than 16 values, cannot exhaustively binarize", gen->domain->attributes->at(thisAttr)->get_name().c_str());
 
         // If the attribute is binary, we check subsetSizes and assess the quality if they are OK
         if (distr.size()==2) {
@@ -446,10 +448,13 @@ PClassifier TTreeSplitConstructor_ExhaustiveBinary::operator()(
             *dis0 = CAST_TO_DISCDISTRIBUTION(distr[valueIndices[0]]);
             *dis1 *= 0;
             vector<int>::const_iterator ii(valueIndices.begin());
+            ii++;
             for(TBoolCount::const_iterator bi(selection.begin()), be(selection.end()); bi!=be; bi++, ii++)
                *(*bi ? dis1 : dis0) += distr[*ii];
+            cont->outerDistribution->setint(0, dis0->abs);
+            cont->outerDistribution->setint(1, dis1->abs);
 
-            if ((dis0->abs<minSubset) || (dis1->abs<minSubset))
+            if ((dis0->abs < minSubset) || (dis1->abs < minSubset))
               continue; // cannot split like that, to few examples in one of the branches
 
             float thisMeas = measure->operator()(cont, classDistribution, apriorClass);
@@ -496,7 +501,7 @@ PClassifier TTreeSplitConstructor_ExhaustiveBinary::operator()(
           bestMapping = mlnew TIntList(distr.size(), -1);
           vector<int>::const_iterator ii = valueIndices.begin();
           bestMapping->at(*(ii++)) = 0;
-          ITERATE(TBoolCount, bi, selection)
+          ITERATE(TBoolCount, bi, bestSelection)
             bestMapping->at(*(ii++)) = *bi ? 1 : 0;
         }
       }
@@ -516,7 +521,9 @@ PClassifier TTreeSplitConstructor_ExhaustiveBinary::operator()(
   if (attribute->noOfValues() == 2) {
     spentAttribute = bestAttr;
     descriptions = mlnew TStringList(attribute.AS(TEnumVariable)->values.getReference());
-    return mlnew TClassifierFromVarFD(attribute, gen->domain, bestAttr, subsetSizes);
+    TClassifierFromVarFD *cfv = mlnew TClassifierFromVarFD(attribute, gen->domain, bestAttr, subsetSizes);
+    cfv->transformUnknowns = false;
+    return cfv;
   }
 
   string s0, s1;
@@ -542,9 +549,11 @@ PClassifier TTreeSplitConstructor_ExhaustiveBinary::operator()(
   descriptions->push_back(ns0>1 ? "in ["+s0+"]" : s0);
   descriptions->push_back(ns1>1 ? "in ["+s1+"]" : s1);
 
-  bvar->name = gen->domain->attributes->at(bestAttr)->name;
+  bvar->set_name(gen->domain->attributes->at(bestAttr)->get_name());
   spentAttribute = (ns0==1) && (ns1==1) ? bestAttr : -1;
-  return mlnew TClassifierFromVarFD(bvar, gen->domain, bestAttr, subsetSizes, mlnew TMapIntValue(bestMapping));
+  TClassifierFromVarFD *cfv = mlnew TClassifierFromVarFD(bvar, gen->domain, bestAttr, subsetSizes, mlnew TMapIntValue(bestMapping));
+  cfv->transformUnknowns = false;
+  return cfv;
 }
 
 
@@ -741,7 +750,9 @@ PClassifier TTreeSplitConstructor_OneAgainstOthers::operator()(
   if (attribute->noOfValues() == 2) {
     spentAttribute = bestAttr;
     descriptions = mlnew TStringList(attribute.AS(TEnumVariable)->values.getReference());
-    return mlnew TClassifierFromVarFD(attribute, gen->domain, bestAttr, subsetSizes);
+    TClassifierFromVarFD *cfv = mlnew TClassifierFromVarFD(attribute, gen->domain, bestAttr, subsetSizes);
+    cfv->transformUnknowns = false;
+    return cfv;
   }
 
   const string &bestValueS = attribute.AS(TEnumVariable)->values->at(bestValue);
@@ -749,13 +760,15 @@ PClassifier TTreeSplitConstructor_OneAgainstOthers::operator()(
   descriptions->push_back(string("not ") + bestValueS);
   descriptions->push_back(bestValueS);
   
-  bvar->name = gen->domain->attributes->at(bestAttr)->name;
+  bvar->set_name(gen->domain->attributes->at(bestAttr)->get_name());
 
   TIntList *bestMapping = mlnew TIntList(attribute.AS(TEnumVariable)->values->size(), 0);
   PIntList wb = bestMapping;
   bestMapping->at(bestValue) = 1;
   spentAttribute = -1;
-  return mlnew TClassifierFromVarFD(bvar, gen->domain, bestAttr, subsetSizes, mlnew TMapIntValue(bestMapping));
+  TClassifierFromVarFD *cfv = mlnew TClassifierFromVarFD(bvar, gen->domain, bestAttr, subsetSizes, mlnew TMapIntValue(bestMapping));
+  cfv->transformUnknowns = false;
+  return cfv;
 }
 
 
@@ -861,9 +874,11 @@ PClassifier TTreeSplitConstructor_Threshold::operator()(
   sprintf(str, ">%3.3f", bestThreshold);
   descriptions->push_back(str);
 
-  bvar->name = gen->domain->attributes->at(bestAttr)->name;
+  bvar->set_name(gen->domain->attributes->at(bestAttr)->get_name());
   spentAttribute = -1;
-  return mlnew TClassifierFromVarFD(bvar, gen->domain, bestAttr, subsetSizes, mlnew TThresholdDiscretizer(bestThreshold));
+  TClassifierFromVarFD *cfv = mlnew TClassifierFromVarFD(bvar, gen->domain, bestAttr, subsetSizes, mlnew TThresholdDiscretizer(bestThreshold));
+  cfv->transformUnknowns = false;
+  return cfv;
 }
 
 
