@@ -20,7 +20,7 @@
 
 
 // to include Python.h before STL defines a template set (doesn't work with VC 6.0)
-#include "garbage.hpp"
+#include "garbage.hpp" 
 
 #include <set>
 #include <stack>
@@ -45,16 +45,43 @@
 #include "orvector.hpp"
 #include "stringvars.hpp"
 
-
-#include "vars.ppp"
-
+#include "vars.hpp"
 
 DEFINE_TOrangeVector_classDescription(PVariable, "TVarList", true, ORANGE_API)
 DEFINE_TOrangeVector_classDescription(PVarList, "TVarListList", true, ORANGE_API)
 
-list<TVariable *> TVariable::allVariables;
+DEFINE_AttributedList_classDescription(TAttributedFloatList, TFloatList)
+DEFINE_AttributedList_classDescription(TAttributedBoolList, TBoolList)
 
 
+#ifdef _MSC_VER_60
+ TClassDescription template TOrangeVector<TValue, false>::st_classDescription;
+ ORANGE_EXTERN template class ORANGE_API TOrangeVector<TValue, false>;
+#else
+ DEFINE_TOrangeVector_classDescription(TValue, "TOrangeVector<TValue, false>", false, ORANGE_API)
+#endif
+
+#include "vars.ppp"
+
+bool mmvDeallocated = 0;
+
+/* The following class is to be used only for the allVariablesMap variable.
+ * It sets a flag after it has been deallocated, so that after program shutdown
+ * the deallocated TVariable instances do not try to remove themselves from the
+ * map. Besides, this class overrides the non-virtual destructor of STL
+ * multimap, so it's not functional if the variable is declared using MMV's
+ * supertype!
+ */
+class MMV :	public multimap<string, TVariable *> {
+public:
+	~MMV() {
+		mmvDeallocated = 1;
+	}
+};
+
+
+MMV TVariable::allVariablesMap;
+
 const char *sortedDaNe[] = {"da", "ne", 0 };
 const char *resortedDaNe[] = {"ne", "da", 0};
 
@@ -78,13 +105,17 @@ TVariable *TVariable::getExisting(const string &name, const int &varType, TStrin
   vector<pair<TVariable *, int> > candidates;
   TStringList::const_iterator vvi, vve;
 
-  ITERATE(list<TVariable *>, vi, TVariable::allVariables) {
-    if (((*vi)->varType == varType) && ((*vi)->name == name)) {
+   pair<MMV::iterator,MMV::iterator> rp = allVariablesMap.equal_range(name);
+   MMV::iterator it;
+ 
+   for (it=rp.first; it!=rp.second; ++it) {	
+     if ((it->second)->varType == varType) {
+
       int tempStat = TVariable::OK;
 
       // non-discrete attributes are always ok,
       // discrete ones need further checking if they have any defined values
-      TEnumVariable *evar = dynamic_cast<TEnumVariable *>(*vi);
+      TEnumVariable *evar = dynamic_cast<TEnumVariable *>(it->second);
       if (evar && evar->values->size()) {
 
         if (fixedOrderValues && !evar->checkValuesOrder(*fixedOrderValues))
@@ -117,7 +148,7 @@ TVariable *TVariable::getExisting(const string &name, const int &varType, TStrin
          }
        }
 
-      candidates.push_back(make_pair(*vi, tempStat));
+      candidates.push_back(make_pair(it->second, tempStat));
       if (tempStat == TVariable::OK)
         break;
     }
@@ -215,6 +246,31 @@ bool TVariable::isEquivalentTo(const TVariable &old) const {
 }
 
 
+void TVariable::registerVariable() {
+  allVariablesMap.insert(pair<string, TVariable *>(name, this));
+}
+
+void TVariable::removeVariable() {
+      pair<MMV::iterator,MMV::iterator> rp = allVariablesMap.equal_range(name);
+	  MMV::iterator it;
+
+      for (it=rp.first; it!=rp.second; ++it) {
+		  if (it->second == this) {
+			  allVariablesMap.erase(it);
+              break;
+          }
+      }
+}
+
+TVariable::~TVariable()
+{
+    /* When the program shuts down, it may happen that the list is destroyed before
+       the variables. We do nothing in this case. */
+    if (!mmvDeallocated) {
+        removeVariable();
+    }
+}
+
 TVariable::TVariable(const int &avarType, const bool &ord)
 : varType(avarType),
   ordered(ord),
@@ -223,7 +279,9 @@ TVariable::TVariable(const int &avarType, const bool &ord)
   DC_value(varType, valueDC),
   DK_value(varType, valueDK),
   defaultMetaId(0)
-{}
+{
+  registerVariable();
+}
 
 
 TVariable::TVariable(const string &aname, const int &avarType, const bool &ord)
@@ -234,7 +292,9 @@ TVariable::TVariable(const string &aname, const int &avarType, const bool &ord)
   DC_value(varType, valueDC),
   DK_value(varType, valueDK),
   defaultMetaId(0)
-{ name = aname; };
+{ name = aname; 
+  registerVariable();
+};
 
 
 const TValue &TVariable::DC() const
@@ -248,7 +308,6 @@ const TValue &TVariable::DK() const
 TValue  TVariable::specialValue(int spec) const
 { return TValue(varType, spec); }
 
-
 /*  Converts a human-readable string, representing the value (as read from the file, for example) to TValue.
     TVariable::str2val_add interprets ? as DK and ~ as DC; otherwise it sets val.varType to varType, other fields
     are left intact.*/
@@ -463,7 +522,7 @@ void TEnumVariable::addValue(const string &val)
       }
 
       if (vi==ve)
-        raiseWarning("is '%s' a continuous attribute unintentionally defined by '%s'?", name.c_str(), values->front().c_str());
+        raiseWarning("is '%s' a continuous attribute unintentionally defined by '%s'?", get_name().c_str(), values->front().c_str());
     }
   }
 }
@@ -524,7 +583,7 @@ void TEnumVariable::str2val(const string &valname, TValue &valu)
     if (vi != valuesTree.end())
       valu = TValue(vi->second);
     else if (!str2special(valname, valu))
-      raiseError("attribute '%s' does not have value '%s'", name.c_str(), valname.c_str());
+      raiseError("attribute '%s' does not have value '%s'", get_name().c_str(), valname.c_str());
   }
 
   else {
@@ -532,7 +591,7 @@ void TEnumVariable::str2val(const string &valname, TValue &valu)
     if (vi!=values->end())
       valu = TValue(int(vi - values->begin()));
     else if (!str2special(valname, valu))
-      raiseError("attribute '%s' does not have value '%s'", name.c_str(), valname.c_str());
+      raiseError("attribute '%s' does not have value '%s'", get_name().c_str(), valname.c_str());
   }
 }
 
@@ -786,7 +845,7 @@ int TFloatVariable::str2val_low(const string &valname, TValue &valu)
 void TFloatVariable::str2val(const string &valname, TValue &valu)
 {
   switch (str2val_low(valname, valu)) {
-    case -1: raiseError("'%s' is not a legal value for continuous attribute '%s'", valname.c_str(), name.c_str());
+    case -1: raiseError("'%s' is not a legal value for continuous attribute '%s'", valname.c_str(), get_name().c_str());
     case -2: raiseError("value %5.3f out of range %5.3f-%5.3f", valu.floatV, startValue, endValue);
   }
 }
@@ -810,3 +869,4 @@ void TFloatVariable::val2str(const TValue &valu, string &vname) const
     vname = buf;
   }
 }
+
