@@ -76,18 +76,18 @@ if 1: ### Definitions of method slots
                 ]
 
   specialsequencemethods=[
-                ("len_sq", "sq_length", "inquiry"),
+                ("len_sq", "sq_length", "lenfunc"),
                 ("concat", "sq_concat", "binaryfunc"),
-                ("repeat", "sq_repeat", "intargfunc"),
-                ("getitem_sq", "sq_item", "intargfunc"),
-                ("getslice", "sq_slice", "intintargfunc"),
-                ("setitem_sq", "sq_ass_item", "intobjargproc"),
-                ("setslice", "sq_ass_slice", "intintobjargproc"),
+                ("repeat", "sq_repeat", "ssizeargfunc"),
+                ("getitem_sq", "sq_item", "ssizeargfunc"),
+                ("getslice", "sq_slice", "ssizessizeargfunc"),
+                ("setitem_sq", "sq_ass_item", "ssizeobjargproc"),
+                ("setslice", "sq_ass_slice", "ssizessizeobjargproc"),
                 ("contains", "sq_contains", "objobjproc")
                 ]
 
   specialmappingmethods=[
-                ("len", "mp_length", "inquiry"),
+                ("len", "mp_length", "lenfunc"),
                 ("getitem", "mp_subscript", "binaryfunc"),
                 ("setitem", "mp_ass_subscript", "objobjargproc")
                 ]
@@ -100,16 +100,16 @@ if 1: ### Definitions of method slots
                
 if 1: ### Definitions of regular expressions
 
-  constrdef_mac=re.compile(r'(?P<constype>ABSTRACT|C_UNNAMED|C_NAMED|C_CALL)\s*\(\s*(?P<typename>\w*)\s*,\s*(?P<basename>\w*\s*)\s*(,\s*"(?P<doc>[^"]*)")?\s*\)')
-  constrdef_mac_call3=re.compile(r'(?P<constype>C_CALL3)\s*\(\s*(?P<typename>\w*)\s*,\s*(?P<callname>\w*\s*)\s*,\s*(?P<basename>\w*\s*)\s*(,\s*"(?P<doc>[^"]*)")?\s*\)')
+  constrdef_mac=re.compile(r'(?P<constype>ABSTRACT|C_UNNAMED|C_NAMED|C_CALL)\s*\(\s*(?P<typename>\w*)\s*(-\s*(?P<displayname>\S*))?,\s*(?P<basename>\w*\s*)\s*(,\s*"(?P<doc>[^"]*)")?\s*\)')
+  constrdef_mac_call3=re.compile(r'(?P<constype>C_CALL3)\s*\(\s*(?P<typename>\w*)\s*(-\s*(?P<displayname>\S*))?,\s*(?P<callname>\w*\s*)\s*,\s*(?P<basename>\w*\s*)\s*(,\s*"(?P<doc>[^"]*)")?\s*\)')
   constrkeywords = re.compile(r'CONSTRUCTOR_KEYWORDS\s*\(\s*(?P<typename>\w*)\s*, \s*"(?P<keywords>[^"]*)"\s*\)')
   nopickle=re.compile(r"NO_PICKLE\s*\(\s*(?P<typename>\w*)\s*\)")
   constrwarndef=re.compile(r"[^\w](ABSTRACT|CONS|C_UNNAMED|C_NAMED|C_CALL)[^\w]")
 
-  datastructuredef=re.compile(r'DATASTRUCTURE\s*\(\s*(?P<typename>\w*)\s*,\s*(?P<structurename>\w*)\s*,\s*(?P<dictfield>\w*)\s*\)')
-  newbasedondef=re.compile(r'(inline)?\s*PyObject\s*\*(?P<typename>\w*)_new\s*\([^)]*\)\s*BASED_ON\s*\(\s*(?P<basename>\w*)\s*,\s*"(?P<doc>[^"]*)"\s*\)\s*(?P<allows_empty_args>ALLOWS_EMPTY)?')
-  basedondef=re.compile(r'BASED_ON\s*\(\s*(?P<typename>\w*)\s*,\s*(?P<basename>\w*)\s*\)')
-  hiddendef=re.compile(r'HIDDEN\s*\(\s*(?P<typename>\w*)\s*,\s*(?P<basename>\w*)\s*\)')
+  datastructuredef=re.compile(r'DATASTRUCTURE\s*\(\s*(?P<typename>\w*)\s*(-\s*(?P<displayname>\S*))?,\s*(?P<structurename>\w*)\s*,\s*(?P<dictfield>\w*)\s*\)')
+  newbasedondef=re.compile(r'(inline)?\s*PyObject\s*\*(?P<typename>\w*)_new\s*\([^)]*\)\s*BASED_ON\s*\(\s*(?P<basename>\w*)\s*(-\s*(?P<displayname>\S*))?,\s*"(?P<doc>[^"]*)"\s*\)\s*(?P<allows_empty_args>ALLOWS_EMPTY)?')
+  basedondef=re.compile(r'BASED_ON\s*\(\s*(?P<typename>\w*)\s*(-\s*(?P<displayname>\S*))?,\s*(?P<basename>\w*)\s*\)')
+  hiddendef=re.compile(r'HIDDEN\s*\(\s*(?P<typename>\w*)\s*(-\s*(?P<displayname>\S*))?,\s*(?P<basename>\w*)\s*\)')
   
 
   allspecial=['('+m[0]+')' for m in specialmethods+specialnumericmethods+specialsequencemethods+specialmappingmethods]
@@ -143,14 +143,34 @@ if 1:
                  "int ccn_func_%(type)s(PyObject *obj, void *ptr) { if (obj == Py_None) { *(GCPtr<T%(type)s> *)(ptr) = GCPtr<T%(type)s>(); return 1; }      if (!PyOr%(type)s_Check(obj)) return 0;      *(GCPtr<T%(type)s> *)(ptr) = PyOrange_As%(type)s(obj); return 1; }\n\n\n"
 
 
+f_underscored = open("../_underscored", "a")
+hump = re.compile("([a-z])([A-Z])")
+
+def camelChange(mo):
+    if mo.end() == len(mo.string) or not 'A'<=mo.string[mo.end()]<='Z':
+        sg = mo.group(2).lower()
+    else:
+        sg = mo.group(2)
+    return "%s_%s" % (mo.group(1), sg)
+    
+def camel2underscore(s):
+    if s[0].isupper():
+        return s
+    else:
+        u = hump.sub(camelChange, s)
+        if u != s:
+            f_underscored.write("%-40s %s\n" % (s, u))
+        return u
+
+
 def detectConstructors(line, classdefs):
   found=constrdef_mac.search(line)
   if not found:
     found=constrdef_mac_call3.search(line)
   if found:
-    typename, basename, constype, doc=found.group("typename", "basename", "constype", "doc")
+    typename, basename, constype, doc, displayname =found.group("typename", "basename", "constype", "doc", "displayname")
     printV2("%s (%s): Macro constructor %s", (typename, basename, constype))
-    addClassDef(classdefs, typename, parsedFile, "basetype", basename)
+    addClassDef(classdefs, typename, parsedFile, "basetype", basename, displayname=displayname)
     if constype=="ABSTRACT":
       classdefs[typename].abstract = True
     else:
@@ -228,7 +248,7 @@ def detectAttrs(line, classdefs):
 
 def detectMethods(line, classdefs):
   # The below if is to avoid methods, like, for instance, map's clear to be recognized
-  # also as a special method. Special methods never include PYARGS...
+  # also as a special method. Special methods never include PYARGS.
   if line.find("PYARGS")<0:
     found=specialmethoddef.search(line)
     if found:
@@ -253,12 +273,16 @@ def detectMethods(line, classdefs):
   if typename:  
     if not classdefs.has_key(typename) and "_" in typename:
       com = typename.split("_")
-      if len(com)==2 and classdefs.has_key(com[0]):
-        typename = com[0]
-        cname = com[1] + "_" + cname
+      for i in range(1, len(com)):
+          subname = "_".join(com[:-i]) 
+          if subname in classdefs:
+              typename = subname
+              cname = "_".join(com[-i:])+"_"+cname
+              break
 
     if not methodname:
-      methodname = cname
+      methodname = camel2underscore(cname)
+
       
     addClassDef(classdefs, typename, parsedFile)
     classdefs[typename].methods[methodname]=MethodDefinition(argkw=argkw, arguments=doc, cname=cname)
@@ -267,30 +291,30 @@ def detectMethods(line, classdefs):
 def detectHierarchy(line, classdefs):
   found=datastructuredef.search(line)
   if found:
-    typename, structurename, dictfield = found.group("typename", "structurename", "dictfield")
-    addClassDef(classdefs, typename, parsedFile, "datastructure", structurename)
+    typename, structurename, dictfield, displayname = found.group("typename", "structurename", "dictfield", "displayname")
+    addClassDef(classdefs, typename, parsedFile, "datastructure", structurename, displayname=displayname)
     addClassDef(classdefs, typename, parsedFile, "dictfield", dictfield, 0)
     printV2("%s: definition/declaration of datastructure", typename)
     return 1
   
   found=newbasedondef.match(line)
   if found:
-    typename, basename, doc = found.group("typename", "basename", "doc")
+    typename, basename, doc, displayname = found.group("typename", "basename", "doc", "displayname")
     allows_empty_args = bool(found.group("allows_empty_args"))
-    addClassDef(classdefs, typename, parsedFile, "basetype", basename, 0)
+    addClassDef(classdefs, typename, parsedFile, "basetype", basename, 0, displayname)
     addClassDef(classdefs, typename, parsedFile, "constructor", ConstructorDefinition(arguments=doc, type="MANUAL", allows_empty_args=allows_empty_args))
     return 1
 
   found=basedondef.match(line)
   if found:
-    typename, basename = found.group("typename", "basename")
-    addClassDef(classdefs, typename, parsedFile, "basetype", basename, 0)
+    typename, basename, displayname = found.group("typename", "basename", "displayname")
+    addClassDef(classdefs, typename, parsedFile, "basetype", basename, 0, displayname)
     return 1
 
   found=hiddendef.match(line)
   if found:
-    typename, basename = found.group("typename", "basename")
-    addClassDef(classdefs, typename, parsedFile, "basetype", basename, 0)
+    typename, basename, displayname = found.group("typename", "basename", "displayname")
+    addClassDef(classdefs, typename, parsedFile, "basetype", basename, 0, displayname=displayname)
     classdefs[typename].hidden = 1
     return 1
 
@@ -387,7 +411,8 @@ def parseFiles():
       if line.strip().startswith("PYXTRACT_IGNORE"):
         continue
       detectHierarchy(line, classdefs) # BASED_ON, DATASTRUCTURE those lines get detected twice!
-      for i in [detectConstructors, detectAttrs, detectMethods, detectCallDoc]:
+      # detectMethods must be before detectAttrs to allow for methods named get_something, set_something
+      for i in [detectConstructors, detectMethods, detectAttrs, detectCallDoc]:
         if i(line, classdefs):
           break
       else:
@@ -546,7 +571,8 @@ def writeAppendix(filename, targetname, classdefs, aliases):
       properties.sort(lambda x,y:cmp(x[0], y[0]))
       outfile.write("PyGetSetDef "+type+"_getset[]=  {\n")
       for (name, definition) in properties:
-        outfile.write('  {"%s"' % name)
+        camelname = camel2underscore(name)
+        outfile.write('  {"%s"' % camelname)
         if definition.hasget:
           outfile.write(", (getter)%s_get_%s" % (type, name))
         else:
@@ -719,7 +745,8 @@ PyObject *%(wholename)s__reduce__(PyObject *self) { return Py_BuildValue("O(s(i)
     outfile.write('PyTypeObject PyOr'+type+'_Type_inh = {\n')
     outfile.write('  PyObject_HEAD_INIT((_typeobject *)&PyType_Type)\n')
     outfile.write('  0,\n')
-    outfile.write('  "%s.%s",\n' % (modulename, type))
+    displayname = getattr(fields, "displayname", "Orange.core."+type) # was: <modulename>.<type>
+    outfile.write('  "%s",\n' % displayname)
     outfile.write('  sizeof(%s), 0,\n' % fields.datastructure)
     innulls=writeslots(specialmethods, 1)
     outfile.write((innulls and '\n' or '') + '};\n\n')
@@ -745,6 +772,14 @@ PyObject *%(wholename)s__reduce__(PyObject *self) { return Py_BuildValue("O(s(i)
   outfile.close()
 
 
+def writeDisplayNames(filename, classdefs):
+  outfile = open("px/%s.new" % filename, "wt")
+  newfiles.append(filename)
+  for classname, fields in classdefs.items():
+    if hasattr(fields, "displayname"):
+        outfile.write('%s\t%s\n' % (classname, fields.displayname))
+
+
 def writeAppendices(classdefs):
   filenamedef=re.compile(r"(?P<stem>.*)\.\w*$")
   for filename in filenames:
@@ -899,8 +934,11 @@ def writeInitialization(functions, constants):
 
   if len(myclasses):
     for classname in ks:
+      exportedname = "OrangeBase" if classname=="Orange" else classname
       if not classdefs[i].hidden:
-        functionsfile.write('     PyModule_AddObject(mod, "%s", (PyObject *)&PyOr%s_Type);\n' % (classname, classname))
+        functionsfile.write('     PyModule_AddObject(mod, "%s", ' \
+                            '(PyObject *)&PyOr%s_Type);\n' %
+                            (exportedname, classname))
 
   for type, fields in myclasses.items():
       if fields.subconstants:
@@ -1054,6 +1092,7 @@ def make():
   if not os.path.isdir("px"):
     os.mkdir("px")
   writeAppendices(classdefs)
+  writeDisplayNames("displaynames", classdefs)
   writeExterns()
   writeInitialization(functions, constants)
   writeGlobals()
