@@ -47,8 +47,8 @@ using namespace std;
  
 #define PyCATCH_r(r) \
   } \
-catch (pyexception err)   { err.restore(); return r; } \
-catch (exception err) { PYERROR(PyExc_StatcKernel, err.what(), r); }
+catch (pyexception& err)   { err.restore(); return r; } \
+catch (exception& err) { PYERROR(PyExc_StatcKernel, err.what(), r); }
 
 PyObject *PyExc_StatcKernel;
 PyObject *PyExc_StatcWarning;
@@ -1452,6 +1452,12 @@ bool getSmootherPars(PyObject *args, vector<pair<double, double> > &points, vect
   }
   Py_DECREF(iter);
   
+  if (nUniquePoints(xpoints) < 2)
+  {
+      PyErr_Format(PyExc_ValueError, "A list with at least 2 unique points required.");
+      return false;
+  }
+
   if (distMethod != -1)
     sampFuncs[distMethod](xpoints, nPoints, sampPoints);
 
